#!/bin/bash

# Script to patch the ch341 usb serial driver module to support
# extra baudrates and status lines (needed for USB thermometers)
# Run with sudo

# WARNING: Please check what this script does before running it!
# Note that it may overwrite existing kernel sources un /usr/src,
# so please make sure you either change the directory variables
# or backup your kernel if needed.

builddir=/usr/src
srcdir=/usr/src
patchdir=`pwd`
patchfile1=extend-ch341.patch
patchfile2=fixids-pl2303.patch
patchedfile=drivers/usb/serial/ch341.c
module=drivers/usb/serial/
modulefile1=drivers/usb/serial/ch341.ko
modulefile2=drivers/usb/serial/pl2303.ko
dest1=/lib/modules/`uname -r`/kernel/$modulefile1
dest2=/lib/modules/`uname -r`/kernel/$modulefile2

apt-get install linux-source	# We need the kernel sources
cd $builddir
tar xvjf $srcdir/linux-source-2.6.*.tar.bz2
cd linux-source-2.6.*
cp /boot/config-$(uname -r) .config	# Get kernel configuration
patch -p1 < $patchdir/$patchfile1	# Patch ch341.c module code
patch -p1 < $patchdir/$patchfile2	# Patch pl2303.* module code
make clean
make oldconfig			# Prepare for compiling
make prepare
make scripts
make M=$module			# Rebuild USB serial modules
cp $dest1 $dest1.old		# Backup old ch341 module
cp $modulefile1 $dest1		# Replace by new ch341.ko
cp $dest2 $dest2.old		# Backup old pl2303 module
cp $modulefile2 $dest2		# Replace by new pl2303.ko
depmod -a			# Rebuild module dependencies

echo "Created patched $dest1 and $dest2"
echo "You can now remove the kernel sources at" /usr/src/linux-source-2.6.??
echo "Don't forget to re-run this script after a kernel update!"

